import java.util.Random;
import java.util.HashMap;
import java.util.HashSet;

public class LifeGame {
	final int rows;
	final int cols;
	final int dataRows;
	final int dataCols;

	final short[][] grid;
	final short[][] alt;

	public LifeGame (int rows, int cols) {

		dataRows = (rows + 3) / 4;
		dataCols = (cols + 3) / 4;

		this.rows = dataRows * 4;
		this.cols = dataCols * 4;

		grid = new short[dataRows + 1][dataCols + 1];
		alt = new short[dataRows + 1][dataCols + 1];

		// wrap around
		grid[dataRows]  = grid[0];
		alt[dataRows] = alt[0];
	}


	public void randomize (long seed) {
		Random random = new Random(seed);

		for (int i = 0; i < dataRows; ++i) {
			for (int j = 0; j < dataCols; ++j) {
				grid[i][j] = (short)( random.nextInt(0xffff) & random.nextInt(0xffff) & random.nextInt(0xffff)& random.nextInt(0xffff)& random.nextInt(0xffff)& random.nextInt(0xffff)& random.nextInt(0xffff)& random.nextInt(0xffff) );
			}
		}
	}

	static String[] hexes = {
		"    ", "   *", "  * ", "  **",
		" *  ", " * *", " ** ", " ***",
		"*   ", "*  *", "* * ", "* **",
		"**  ", "** *", "*** ", "****",
	};

	public String toString () {
		final StringBuilder buffer = new StringBuilder();

		for (int i = 0; i < dataRows; ++i) {
			final short[] row = grid[i];

			for (int rank = 0; rank < 16; rank += 4) {

				for (int j = 0; j < dataCols; ++j) {
					buffer.append(hexes[ (row[j] >> rank) & 0xf ]);
				}

				buffer.append("\n");
			}
		}

		return buffer.toString();
	}

	public boolean isDead () {
		for (int i = 0; i < dataRows; ++i) {
			final short[] row = grid[i];

			for (int j = 0; j < dataCols; ++j) {
				if ( row[j] != 0 ) {
					return false;
				}
			}
		}

		return true;
	}

	static final char[] hexChars = "0123456789abcdef".toCharArray();

	public String toHexString () {
		char[] characters = new char[dataRows * dataCols * 4];
		int k  = -1;

		for (int i = 0; i < dataRows; ++i) {
			final short[] row = grid[i];

			for (int j = 0; j < dataCols; ++j) {
				characters[++k] = hexChars[(row[j] >> 12) & 0xf];
				characters[++k] = hexChars[(row[j] >>  8) & 0xf];
				characters[++k] = hexChars[(row[j] >>  4) & 0xf];
				characters[++k] = hexChars[(row[j]      ) & 0xf];
			}
		}

		return new String(characters);
	}

	public void step () {
		// wrap horizontally
		for (int i = 0; i <= dataRows; ++i) {
			grid[i][dataCols] = grid[i][0];
		}

		// working grid
		int work[][] = new int[6][6];

		short[] prevRow = grid[dataRows - 1];

		for (int i = 0; i < dataRows; ++i) {
			short[] currRow   = grid[i];
			short[] nextRow   = grid[i + 1];
			short[] targetRow = alt[i];

			short   prevBankPrevRow = prevRow[dataCols - 1];
			short   prevBankCurrRow = currRow[dataCols - 1];
			short   prevBankNextRow = nextRow[dataCols - 1];

			for (int j = 0; j < dataCols; ++j) {
				short   currBankPrevRow = prevRow[j];
				short   currBankCurrRow = currRow[j];
				short   currBankNextRow = nextRow[j];

				short   nextBankPrevRow = prevRow[j + 1];
				short   nextBankCurrRow = currRow[j + 1];
				short   nextBankNextRow = nextRow[j + 1];

				//~ System.out.printf("%04x %04x %04x\n", prevBankPrevRow, currBankPrevRow, nextBankPrevRow);
				//~ System.out.printf("%04x %04x %04x\n", prevBankCurrRow, currBankCurrRow, nextBankCurrRow);
				//~ System.out.printf("%04x %04x %04x\n", prevBankNextRow, currBankNextRow, nextBankNextRow);

				int   result = 0;

				// set up work array
				{
					work[0][0] = ((prevBankPrevRow & 0x1000) != 0) ? 1 : 0;
					work[0][1] = ((currBankPrevRow & 0x8000) != 0) ? 1 : 0;
					work[0][2] = ((currBankPrevRow & 0x4000) != 0) ? 1 : 0;
					work[0][3] = ((currBankPrevRow & 0x2000) != 0) ? 1 : 0;
					work[0][4] = ((currBankPrevRow & 0x1000) != 0) ? 1 : 0;
					work[0][5] = ((nextBankPrevRow & 0x8000) != 0) ? 1 : 0;

					work[1][0] = ((prevBankCurrRow & 0x0001) != 0) ? 1 : 0;
					work[1][1] = ((currBankCurrRow & 0x0008) != 0) ? 1 : 0;
					work[1][2] = ((currBankCurrRow & 0x0004) != 0) ? 1 : 0;
					work[1][3] = ((currBankCurrRow & 0x0002) != 0) ? 1 : 0;
					work[1][4] = ((currBankCurrRow & 0x0001) != 0) ? 1 : 0;
					work[1][5] = ((nextBankCurrRow & 0x0008) != 0) ? 1 : 0;

					work[2][0] = ((prevBankCurrRow & 0x0010) != 0) ? 1 : 0;
					work[2][1] = ((currBankCurrRow & 0x0080) != 0) ? 1 : 0;
					work[2][2] = ((currBankCurrRow & 0x0040) != 0) ? 1 : 0;
					work[2][3] = ((currBankCurrRow & 0x0020) != 0) ? 1 : 0;
					work[2][4] = ((currBankCurrRow & 0x0010) != 0) ? 1 : 0;
					work[2][5] = ((nextBankCurrRow & 0x0080) != 0) ? 1 : 0;

					work[3][0] = ((prevBankCurrRow & 0x0100) != 0) ? 1 : 0;
					work[3][1] = ((currBankCurrRow & 0x0800) != 0) ? 1 : 0;
					work[3][2] = ((currBankCurrRow & 0x0400) != 0) ? 1 : 0;
					work[3][3] = ((currBankCurrRow & 0x0200) != 0) ? 1 : 0;
					work[3][4] = ((currBankCurrRow & 0x0100) != 0) ? 1 : 0;
					work[3][5] = ((nextBankCurrRow & 0x0800) != 0) ? 1 : 0;

					work[4][0] = ((prevBankCurrRow & 0x1000) != 0) ? 1 : 0;
					work[4][1] = ((currBankCurrRow & 0x8000) != 0) ? 1 : 0;
					work[4][2] = ((currBankCurrRow & 0x4000) != 0) ? 1 : 0;
					work[4][3] = ((currBankCurrRow & 0x2000) != 0) ? 1 : 0;
					work[4][4] = ((currBankCurrRow & 0x1000) != 0) ? 1 : 0;
					work[4][5] = ((nextBankCurrRow & 0x8000) != 0) ? 1 : 0;

					work[5][0] = ((prevBankNextRow & 0x0001) != 0) ? 1 : 0;
					work[5][1] = ((currBankNextRow & 0x0008) != 0) ? 1 : 0;
					work[5][2] = ((currBankNextRow & 0x0004) != 0) ? 1 : 0;
					work[5][3] = ((currBankNextRow & 0x0002) != 0) ? 1 : 0;
					work[5][4] = ((currBankNextRow & 0x0001) != 0) ? 1 : 0;
					work[5][5] = ((nextBankNextRow & 0x0008) != 0) ? 1 : 0;
				}

				//~ for (int x = 0; x < 6; ++x) {
					//~ for (int y = 0; y < 6; ++y) {
						//~ System.out.print(work[x][y]);
					//~ }
					//~ System.out.println();
				//~ }

				//~ System.out.println();

				// count neighbours
				for (int x = 0; x < 4; ++x) {
					for (int y = 0; y < 4; ++y) {
						int count = 	work[x  ][y  ] + work[x+1][y  ] + work[x+2][y  ] +
								work[x  ][y+1]                  + work[x+2][y+1] +
								work[x  ][y+2] + work[x+1][y+2] + work[x+2][y+2];

						if (count >= 2 && count <= 3) {
							result |= 1 << (4 * x + (3 - y));
						}

						//~ work[x][y] = count;
					}
				}

				//~ for (int x = 0; x < 4; ++x) {
					//~ System.out.print(" ");
					//~ for (int y = 0; y < 4; ++y) {
						//~ System.out.print(work[x][y]);
					//~ }
					//~ System.out.print(" ");
					//~ for (int y = 0; y < 4; ++y) {
						//~ System.out.print((work[x][y] >= 2 && work[x][y] <= 3) ? 1 : 0);
					//~ }
					//~ System.out.println();
				//~ }

				//~ System.out.printf(" result %04x %s\n", result, Integer.toString(result, 2));

				// store result
				alt[i][j] = (short)result;

				// next cycle
				prevBankPrevRow = currBankPrevRow;
				prevBankCurrRow = currBankCurrRow;
				prevBankNextRow = currBankNextRow;
			}

			prevRow = currRow;
		}

		for (int i = 0; i <= dataRows; ++i) {
			final short[] temp = grid[i];
			grid[i] = alt[i];
			alt[i] = temp;
		}
	}

	public static void main (String...args) throws Exception {
		int rows = args.length > 0 ? Integer.parseInt(args[0]) : 28;
		int cols = args.length > 1 ? Integer.parseInt(args[1]) : 124;
		int seed = args.length > 2 ? Integer.parseInt(args[2]) : 1;
		final int max_i = args.length > 3 ? Integer.parseInt(args[3]) : 1000000;

		LifeGame game = new LifeGame(rows, cols);

		int period = -1;

		find_good_seed:
		for (;;) {
			game.randomize(seed);

			System.out.println("Seed: " + seed);
			System.out.println(game);

			HashMap<String, Integer> states = new HashMap<String, Integer>();

			for (int i = 0; i < max_i; ++i) {
				if (game.isDead()) {
					System.out.println("Seed " + seed + " dies after " + i);
					++seed;
					continue find_good_seed;
				}

				String hex = game.toHexString();

				Integer prevStep = states.get(hex);

				if (prevStep != null) {
					period = (i - prevStep);
					System.out.println("period " + period + " after " + i);
					break find_good_seed;
				} else {
					states.put(hex, i);
				}

				game.step();
			}

			System.out.println("Not periodic after " + max_i + " steps.");

			break;
		}

		for (;;) {
			game.step();
			System.out.println("--------------- seed: " + seed + " period: " + (period > 0 ? String.valueOf(period) : "?"));
			System.out.println(game);

			Thread.sleep(200);
		}
	}
}
